/*
 * File:       ControlCenter.C
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    Mon Apr 21 1997
 * Revision:   $Revision: 1.4 $
 * Updated:    $Date: 1998/04/25 06:14:58 $
 * Copyright 1997 NPAC, Syracuse University
 *
 */

#include "ControlCenter.h"

#include <stdlib.h>
#include <fstream.h>
#include <X11/xpm.h>

#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/BulletinB.h>

#include <Proto.h>

void ControlCenter::
launchCB( Widget, ControlCenter* main, XtPointer )
{
   ACE_TRACE( "(ControlCenter::launchCB)" );
   main->spawnNewViewer();
}

void ControlCenter::
quitCB( Widget, ControlCenter*, XtPointer )
{
   ACE_DEBUG(( LM_DEBUG, "Quiting Control Center\n" ));
   exit( 0 );
}

void ControlCenter::
mapCB( Widget w, ControlCenter* main, XmDrawingAreaCallbackStruct* cbs )
{
   ACE_TRACE( "(ControlCenter::mapCB)" );
   static unsigned short beg_x, beg_y;
   static Window map_win = XtWindow( w );
   static bool sensitive = false;
   
   if( main->mode == MASTER )
   {
      XEvent* event = cbs->event;
      XButtonEvent& bevent = cbs->event->xbutton;
   
      if( event->xany.type == ButtonPress )
      {
	 XSetForeground( main->disp, main->map_gc, main->gridColor.pixel );
	 XDrawRectangle( main->disp, map_win, main->map_gc,
			 main->area_ulx, main->area_uly, main->area_width, main->area_height );

	 beg_x = bevent.x;
	 beg_y = bevent.y;

	 main->area_ulx = beg_x / main->chunkHorPix * main->chunkHorPix;
	 main->area_uly = beg_y / main->chunkVerPix * main->chunkVerPix;
	 main->area_width = ( 1 + beg_x / main->chunkHorPix ) * main->chunkHorPix
	    - main->area_ulx;
	 main->area_height = ( 1 + beg_y / main->chunkVerPix ) * main->chunkVerPix
	    - main->area_uly;
	 XSetForeground( main->disp, main->map_gc, main->selColor.pixel );
	 XDrawRectangle( main->disp, map_win, main->map_gc,
			 main->area_ulx, main->area_uly, main->area_width, main->area_height );
	 sensitive = true;
      }
      else if( event->xany.type == ButtonRelease )
      {
	 if( sensitive )
	 {
	    //info on selected area: ulLon, ulLat, szLon, szLat
	    unsigned short selInfo[4];
	    selInfo[0] = main->nwLon - main->area_ulx / main->chunkHorPix * main->chunkSzMin;
	    selInfo[1] = main->nwLat - main->area_uly / main->chunkVerPix * main->chunkSzMin;
	    selInfo[2] = main->area_width / main->chunkHorPix * main->chunkSzMin;
	    selInfo[3] = main->area_height / main->chunkVerPix * main->chunkSzMin;

	    ACE_DEBUG(( LM_DEBUG, "ulLon=%.2f, ulLatn=%.2f, szLon=%d, szLat=%d\n",
			selInfo[0] / 60.0, selInfo[1] / 60.0, selInfo[2], selInfo[3] ));

	    MsgHeader<VIEWER_SAP> hd( NWRL, MsgHeader<VIEWER_SAP>::BROADCAST, sizeof selInfo );
	    ACE_Message_Block mb( ( const char* )selInfo, sizeof selInfo );
	    main->sendToTango( hd, mb, 0 );
	    main->mainLink->sendSelArea();
	    sensitive = false;
	 }
      }
      else
      {
	 XSetForeground( main->disp, main->map_gc, main->gridColor.pixel );
	 XDrawRectangle( main->disp, map_win, main->map_gc,
			 main->area_ulx, main->area_uly, main->area_width, main->area_height );

	 if( beg_x <= bevent.x )
	    main->area_width = ( 1 + ( ( bevent.x < main->bitmapWidth ) ? bevent.x
				       : main->bitmapWidth - 1 ) / main->chunkHorPix )
	       * main->chunkHorPix - main->area_ulx;
	 else
	 {
	    main->area_ulx = ( ( bevent.x > 0 ) ? bevent.x : 0 ) / main->chunkHorPix
	       * main->chunkHorPix;
	    main->area_width = ( 1 + beg_x / main->chunkHorPix ) * main->chunkHorPix
	       - main->area_ulx;
	 }

	 if( beg_y <= bevent.y )
	    main->area_height = ( 1 + ( ( bevent.y < main->bitmapHeight ) ? bevent.y
					: main->bitmapHeight - 1 ) / main->chunkVerPix )
	       * main->chunkVerPix - main->area_uly;
	 else
	 {
	    main->area_uly = ( ( bevent.y > 0 ) ? bevent.y : 0 ) / main->chunkVerPix
	       * main->chunkVerPix;
	    main->area_height = ( 1 + beg_y / main->chunkVerPix ) * main->chunkVerPix
	       - main->area_uly;
	 }
      
	 XSetForeground( main->disp, main->map_gc, main->selColor.pixel );
	 XDrawRectangle( main->disp, map_win, main->map_gc,
			 main->area_ulx, main->area_uly, main->area_width, main->area_height );
      }
      ACE_DEBUG(( LM_DEBUG, "loc [%d, %d] beg_grid [%d, %d], end_grid [%d, %d]\n",
		  bevent.x, bevent.y,
		  main->area_ulx, main->area_uly,
		  main->area_width, main->area_height ));
   }
}

void ControlCenter::
exposeCB( Widget w, ControlCenter* main, XtPointer )
{
   ACE_TRACE( "(ControlCenter::exposeCB)" );
   static Window map_win = XtWindow( w );
   
   XSetForeground( main->disp, main->map_gc, main->gridColor.pixel );
   for( int i=0; i<=main->bitmapHeight; i+=main->chunkVerPix )
      XDrawLine( main->disp, map_win, main->map_gc, 0, i, main->bitmapWidth, i );
   
   for( i=0; i<=main->bitmapWidth; i+=main->chunkHorPix )
      XDrawLine( main->disp, map_win, main->map_gc, i, 0, i, main->bitmapHeight );

   XSetForeground( main->disp, main->map_gc, main->selColor.pixel );
   XDrawRectangle( main->disp, map_win, main->map_gc,
		   main->area_ulx, main->area_uly, main->area_width, main->area_height );
}

void ControlCenter::
setSelArea( const AreaParams& ap )
{
   ACE_TRACE( "(ControlCenter::setSelArea)" );
   Window map_win = XtWindow( map_widget );
   XSetForeground( disp, map_gc, gridColor.pixel );
   XDrawRectangle( disp, map_win, map_gc, area_ulx, area_uly, area_width, area_height );

   area_ulx = ap[0];
   area_uly = ap[1];
   area_width = ap[2];
   area_height = ap[3];

   XSetForeground( disp, map_gc, selColor.pixel );
   XDrawRectangle( disp, map_win, map_gc, area_ulx, area_uly, area_width, area_height );
}

int ControlCenter::
paramsInit( void )
{
   ACE_TRACE( "(ControlCenter::paramsInit)" );
   if( !( pathToGIS = getenv( "GIS_VIEWER_PATH" ) ) )
   {
      fprintf( stderr, "Cannot find GIS_VIEWER_PATH\n" );
      ACE_ERROR(( LM_ERROR,
		  "[!](ControlCenter::paramsInit) Cannot find GIS_VIEWER_PATH\n" ));
      exit( 1 );
   }
	    
   if( !( pathToMap = getenv( "GIS_MAP_PATH" ) ) )
   {
      displayMap = false;
      fprintf( stderr, "Cannot find GIS_MAP_PATH\n" );
      ACE_ERROR(( LM_ERROR,
		  "[!](ControlCenter::paramsInit) Cannot find GIS_MAP_PATH\n" ));
   }

   struct mapParam
   {
	 const char* paramName;
	 unsigned short* paramValue;
   };
	 
   mapParam mapParamTable[] = 
   {
      { "GIS_MAP_CHSIZE", &chunkSzMin },
      { "GIS_MAP_CHHPIX", &chunkHorPix },
      { "GIS_MAP_CHVPIX", &chunkVerPix },
      { "GIS_MAP_NWLON", &nwLon },
      { "GIS_MAP_NWLAT", &nwLat }
   };
	 
   const char* param;
   for( int i=0; i< sizeof mapParamTable/sizeof mapParamTable[0]; i++ )
   {
      if( param = getenv( mapParamTable[i].paramName ) )
	 *mapParamTable[i].paramValue = atoi( param );
      else
      {
	 displayMap = false;
	 fprintf( stderr, "Cannot find %s\n", mapParamTable[i].paramName );
	 ACE_ERROR(( LM_ERROR, "[!](ControlCenter::paramsInit) Cannot find %s\n",
		     mapParamTable[i].paramName ));
      }
   }
   nwLon *= 60;
   nwLat *= 60;

   return 0;
}

int ControlCenter::
guiInit( int argc, char* argv[] )
{
   ACE_TRACE( "(ControlCenter::guiInit)" );
   String fallbacks[] =
   {
      "ControlCenter*background: grey",
      "ControlCenter*fontList:-*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*",
      NULL
   };

   top_level = XtVaAppInitialize( NULL, "ControlCenter", NULL, 0,
				  &argc, argv, fallbacks, NULL );

   Widget rowcol = XtVaCreateManagedWidget( "rowcolumn",
					    xmRowColumnWidgetClass,
					    top_level,
					    NULL );
   if( displayMap )
   {
      Pixmap pixmap;
      XpmAttributes attrs;
      attrs.closeness = 20000;
      attrs.valuemask = XpmSize | XpmCloseness;
      int status = XpmReadFileToPixmap( XtDisplay( top_level ),
					XRootWindowOfScreen( XtScreen( top_level ) ),
					( char* )pathToMap, &pixmap, NULL, &attrs );
      if( status == XpmSuccess )
      {
	 String translations =
	    "<Btn1Motion>: DrawingAreaInput() ManagerGadgetButtonMotion() \n\
<Btn1Down>: DrawingAreaInput() ManagerGadgetArm() \n\
<Btn1Up>: DrawingAreaInput() ManagerGadgetActivate()";
      
	 bitmapWidth = attrs.width;
	 bitmapHeight = attrs.height;

	 map_widget = XtVaCreateManagedWidget(
	    "NYSmap", xmDrawingAreaWidgetClass, rowcol,
	    XmNtranslations, XtParseTranslationTable( translations ),
	    XmNbackgroundPixmap, pixmap,
	    XmNwidth, bitmapWidth+1,
	    XmNheight, bitmapHeight+1,
	    XmNresizePolicy, XmRESIZE_NONE,
	    NULL );

      
	 disp = XtDisplay( map_widget );
	 scr = DefaultScreen( disp );
	 cmap = DefaultColormap( disp, scr );
	 map_gc = XCreateGC( disp, RootWindowOfScreen( XtScreen( map_widget ) ), 0, NULL );
      
	 if( ! XAllocNamedColor( disp, cmap, "red", &unusedColor, &selColor ) )
	    ACE_ERROR(( LM_ERROR, "[!](ControlCenter::guiInit) selColor not allocated\n" ));

	 if( ! XAllocNamedColor( disp, cmap, "grey", &unusedColor, &gridColor ) )
	    ACE_ERROR(( LM_ERROR, "[!](ControlCenter::guiInit) selColor not allocated\n" ));

	 XtAddCallback( map_widget, XmNexposeCallback, ( XtCallbackProc )exposeCB, this );
	 XtAddCallback( map_widget, XmNinputCallback, ( XtCallbackProc )mapCB, this );
      }
      else
	 ACE_ERROR(( LM_ERROR, "[!](ControlCenter::guiInit) %s: %s\n",
		     pathToMap, XpmGetErrorString( status ) ));
   }
   
   launch_button = XtVaCreateManagedWidget( "Launch", xmPushButtonGadgetClass, rowcol,
					    XmNsensitive, FALSE,
					    NULL );
   XtAddCallback( launch_button, XmNactivateCallback, ( XtCallbackProc )launchCB, this );
   
   Widget quit = XtVaCreateManagedWidget( "Quit", xmPushButtonGadgetClass, rowcol, NULL );
   XtAddCallback( quit, XmNactivateCallback, ( XtCallbackProc )quitCB, this );
   return 0;
}

int ControlCenter::
spawnNewViewer( ViewerLinkID vid )
{
   ACE_TRACE( "(ControlCenter::spawnNewViewer)" );
   pid_t chldPid;
   int fd[2];

   if( vid )
   {
      if( viewerIDRep.setID( vid ) == IDrep<ViewerLinkID>::ID_NOROOM )
	 ACE_ERROR_RETURN(( LM_ERROR,
			    "[!](ControlCenter::spawnNewViewer) vid: %d already set\n", vid ),
			  -1 );
   }
   else
      if( viewerIDRep.newID( vid ) == IDrep<ViewerLinkID>::ID_NOROOM )
	 ACE_ERROR_RETURN(( LM_ERROR, "[!](ControlCenter::spawnNewViewer) no more vids\n" ),
			  -1 );
   
   if( mode == MASTER )
      mainLink->viewerSpawned( vid );
   
   if( socketpair( AF_UNIX, SOCK_STREAM, 0, fd ) < 0 )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](ControlCenter::spawnNewViewer) socketpair: %s\n",
			 strerror( errno ) ), -1 );
   
   switch( chldPid = fork() ) 
   { 
      case 0:
	 do { close( fd[1]-- ); } while( fd[0]<fd[1] );
	 char sockstr[4];
	 sprintf( sockstr, "%d", fd[0] );
	 char vidstr[6];
	 sprintf( vidstr, "%d", vid );
	 ACE_DEBUG(( LM_DEBUG, "sock: %d, vid: %d\n", fd[0], vid ));
	 if( execl( pathToGIS,
		    "tweety",
		    sockstr,
		    vidstr,
		    mode == MASTER ? "m" : "s",
		    NULL ) < 0 )
	    ACE_ERROR_RETURN(( LM_ERROR, "[!](ControlCenter::spawnNewViewer) execl: %s%a\n",
			       strerror( errno ), 1 ), -1 );
	 break;

      case -1:
	 ACE_ERROR_RETURN(( LM_ERROR, "[!](ControlCenter::spawnNewViewer) fork: %s\n",
			    strerror( errno ) ), -1 );

      default:
	 close( fd[0] );
	 ViewerLinkHandler* vlh;
	 TRACE_NEW_2( vlh, ViewerLinkHandler( *this, fd[1] ) );
	 if( ! vlh )
	 {
	    close( fd[1] );
	    viewerIDRep.deleteID( vid );
	    ACE_ERROR_RETURN(( LM_ERROR,
			       "[!](ControlCenter::spawnNewViewer) cannot allocate\n" ), -1 );
	 }
	 if( viewerLinkRep.bind( vid, vlh ) )
	 {
	    close( fd[1] );
	    viewerIDRep.deleteID( vid );
	    TRACE_DELETE( vlh );
	    ACE_ERROR_RETURN(( LM_ERROR,
			       "[!](ControlCenter::spawnNewViewer) Bind error\n" ), -1 );
	 }
	 reactor->register_handler( vlh, ACE_Event_Handler::READ_MASK );
	 if( mode == MASTER )
	    viewerSynch( vid );

	 break;
   }
   return 0;
}

void ControlCenter::
viewerClosed( ViewerLinkHandler* vlh )
{
   ACE_TRACE( "(ControlCenter::viewerClosed)" );
   reactor->remove_handler( vlh, ACE_Event_Handler::READ_MASK );
   ViewerLinks::ITERATOR it( viewerLinkRep );
   ViewerLinks::ENTRY* ent;
   while( it.next( ent ) )
   {
      if( ent->int_id_ == vlh )
      {
	 //send notification to slaves
	 if( mode == MASTER )
	    mainLink->viewerClosed( ent->ext_id_ );

	 viewerIDRep.deleteID( ent->ext_id_ );
	 viewerLinkRep.unbind( ent->ext_id_ );
	 break;
      }
      it.advance();
   }
   TRACE_DELETE( vlh );
}

void ControlCenter::
viewerClosed( const ViewerLinkID& vid )
{
   ACE_TRACE( "(ControlCenter::viewerClosed2)" );
   ACE_DEBUG(( LM_DEBUG, "vid: %d\n", vid ));
   ViewerLinkHandler* vlh;
   if( viewerLinkRep.find( vid, vlh ) != -1 )
   {
      reactor->remove_handler( vlh, ACE_Event_Handler::READ_MASK );
      viewerIDRep.deleteID( vid );
      viewerLinkRep.unbind( vid );
      TRACE_DELETE( vlh );
   }
   else
      ACE_ERROR(( LM_ERROR, "[!](ControlCenter::viewerClosed2) vid: %d not found\n",
		  vid ));
}

int ControlCenter::
sendToViewers( MsgHeader<TANGO_SAP>& header, ACE_Message_Block& mb )
{
   ACE_TRACE( "(ControlCenter::sendToViewers)" );
   ViewerLinks::ITERATOR it( viewerLinkRep );
   ViewerLinks::ENTRY* ent;
   while( it.next( ent ) )
   {
      ent->int_id_->send_msg( ( MsgHeader<VIEWER_SAP>& )header, mb );
      it.advance();
   }
   return 0;
}

int ControlCenter::
sendToTango( MsgHeader<VIEWER_SAP>& header, ACE_Message_Block& mb, ViewerLinkHandler* vlh )
{
   ACE_TRACE( "(ControlCenter::sendToTango)" );
   mainLink->send_msg( ( MsgHeader<TANGO_SAP>& )header, mb );
   ViewerLinks::ITERATOR it( viewerLinkRep );
   ViewerLinks::ENTRY* ent;
   while( it.next( ent ) )
   {
      if( ent->int_id_ != vlh )
	 ent->int_id_->send_msg( header, mb );
      
      it.advance();
   }
   return 0;
}

int ControlCenter::
viewerSynch( const ViewerLinkID& vid )
{
   ACE_TRACE( "(ControlCenter::viewerSynch)" );
   ViewerLinks::ITERATOR it( viewerLinkRep );
   ViewerLinks::ENTRY* ent;

   if( viewerLinkRep.current_size() == 1 )
   {
      const signed char nullInfo[] = { -1, 0 };
      it.next( ent );
      MsgHeader<VIEWER_SAP> hd( SYNCI, ent->ext_id_, sizeof  nullInfo );
      ACE_Message_Block mb( ( const char* )nullInfo, sizeof nullInfo );
      sendToTango( hd, mb, 0 );
   }
   else
      while( it.next( ent ) )
      {
	 if( ent->ext_id_ != vid )
	 {
	    MsgHeader<VIEWER_SAP> hd( SYNCR, ent->ext_id_, sizeof vid );
	    ACE_Message_Block mb( ( const char* )&vid, sizeof vid );
	    ent->int_id_->send_msg( hd, mb );
	    break;
	 }
	 it.advance();
      }

   return 0;
}

void ControlCenter::
sendSynchro( void )
{
   ACE_TRACE( "(ControlCenter::sendSynchro)" );
   if( mode == MASTER )
   {
      ViewerLinks::ITERATOR it( viewerLinkRep );
      ViewerLinks::ENTRY* ent;
      while( it.next( ent ) )
      {
	 ViewerLinkID vid = ent->ext_id_;
	 MsgHeader<VIEWER_SAP> hd( SYNCR, ent->ext_id_, sizeof vid );
	 ACE_Message_Block mb( ( const char* )&vid, sizeof vid );
	 ent->int_id_->send_msg( hd, mb );
	 it.advance();
      }
      
      mainLink->sendSynchro();
   }
}
