/*
 * File         : IDrep.h
 * Author       : Pawel Roman
 * Created      : Mon Jul 28 14:33:03 1997
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1997/08/10 22:49:33 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __IDREP_H
#define __IDREP_H

#include <strings.h>

#include<iostream.h>

template<class T>
class IDrep
{
   public:
      enum IDRESULT { ID_OK, ID_NOROOM, ID_NONEX };
      
      IDrep();
      IDrep& operator=( const T& );
      IDRESULT newID( T& id );
      IDRESULT setID( T id );
      IDRESULT isSetID( T id );
      IDRESULT deleteID( T id );
      unsigned short space();
      const T& bitvec();
	 
   private:
      T bitvec_;
};

template<class T>
inline IDrep<T>::
IDrep()
{
   bzero( &bitvec_, sizeof bitvec_ );
}

template<class T>
inline IDrep<T>& IDrep<T>::
operator=( const T& newBitvec )
{
   bitvec_ = newBitvec;
   return *this;
}

template<class T>
inline IDrep<T>::IDRESULT IDrep<T>::
newID( T& id )
{
   id = 1;
   for( int i=0; i<8*sizeof bitvec_; i++ )
   {
      if( !( bitvec_ & id ) )
      {
	 bitvec_ |= id;
	 return ID_OK;
      }
      id <<= 1;
   }
   return ID_NOROOM;
}

template<class T>
inline IDrep<T>::IDRESULT IDrep<T>::
setID( T id )
{
   if( !( bitvec_ & id ) )
   {
      bitvec_ |= id;
      return ID_OK;
   }
   else
      return ID_NOROOM;
}

template<class T>
inline IDrep<T>::IDRESULT IDrep<T>::
isSetID( T id )
{
   return ( bitvec_ & id ) ? ID_OK : ID_NONEX;
}

template<class T>
inline IDrep<T>::IDRESULT IDrep<T>::
deleteID( T id )
{
   if( bitvec_ & id )
   {
      bitvec_ &= ~id;
      return ID_OK;
   }
   else
      return ID_NONEX;
}

template<class T>
inline unsigned short IDrep<T>::
space()
{
   return sizeof bitvec_ * 8;
}

template<class T>
inline const T& IDrep<T>::
bitvec()
{
   return bitvec_;
}

#endif
