/*
 * File:       TangoLinkHandler.C
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    Mon Apr 21 1997
 * Revision:   $Revision: 1.6 $
 * Updated:    $Date: 1998/07/07 12:21:24 $
 * Copyright 1997 NPAC, Syracuse University
 *
 */

#include "TangoLinkHandler.h"
#include "ControlCenter.h"
#include "msgVSPWD.h"
#include "msgVKILD.h"
#include "msgSYNCR.h"
#include "msgSYNCI.h"
#include "msgAUPDT.h"
#include "msgFOCUS.h"

#include <ace/Message_Block.h>

TangoLinkHandler::
TangoLinkHandler( ControlCenter& ctrl, int handle )
   : control( ctrl ), proto( Protocol::SAP(), MSGMAX )
{
   ACE_TRACE( "(TangoLinkHandler::TangoLinkHandler)" );
   proto.getSAP().set_handle( handle );
   control.setMode( ( proto.askMode() == Protocol::P_MASTER ) ?
		    ControlCenter::MASTER : ControlCenter::SLAVE );

   // Message registration
   proto.addMsgType( VSPWD, new msgVSPWD<Header>( &ctrl ) );
   proto.addMsgType( VKILD, new msgVKILD<Header>( &ctrl ) );
   proto.addMsgType( CSYNCR, new msgCSYNCR<Header>( &ctrl ) );
   proto.addMsgType( CSYNCI, new msgCSYNCI<Header>( &ctrl ) );
   proto.addMsgType( AUPDT, new msgAUPDT<Header>( &ctrl ) );
   proto.addMsgType( FOCUS, new msgFOCUS<Header>( &ctrl ) );

   // Synchronization request
   proto.sendMsg( CSYNCR );
}

int TangoLinkHandler::
handle_input( ACE_HANDLE )
{
   ACE_TRACE( "(TangoLinkHandler::handle_input)" );
   if( proto.processAllMsgs( 0,( Protocol::MessageHandler* )viewerMsgHandler, &control )
       == Protocol::P_CLOSED )
      control.connClosed();
   
   return 0;
}


TangoLinkHandler::Protocol::PSTATUS TangoLinkHandler::
viewerMsgHandler( Protocol::SAP& sap, Header& header, ControlCenter* control )
{
   ACE_TRACE( "(TangoLinkHandler::viewerMsgHandler)" );
   Header::LengthType msg_len = header.length();
   ACE_Message_Block mb( msg_len );
   if( sap.recv_n( mb.rd_ptr(), msg_len ) == msg_len )
   {
      control->sendToViewers( header, mb );
      return Protocol::P_OK;
   }
   else
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](TangoLinkHandler::viewerMsgHandler) recv_n error\n" ),
		       Protocol::P_CLOSED );
}

void TangoLinkHandler::
ping( void )
{
   ACE_TRACE( "(TangoLinkHandler::ping)" );
}

int TangoLinkHandler::
send_msg( Header& header, ACE_Message_Block& mb )
{
   ACE_TRACE( "(TangoLinkHandler::send_msg)" );
   Protocol::SAP& sap = proto.getSAP();
   if( header.write( sap ) == MsgHeader<IPC_SAP>::LINK_OK )
   {
      if( sap.send_n( mb.rd_ptr(), header.length() ) != mb.size() )
	 ACE_ERROR_RETURN(( LM_ERROR,
			    "[!](TangoLinkHandler::send_msg) send_n error\n" ), -1 );
   }
   else
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](TangoLinkHandler::send_msg) header.write error\n" ), -1 );
   
   return 0;
}

int TangoLinkHandler::
viewerSpawned( const Header::VID& vid )
{
   ACE_TRACE( "(TangoLinkHandler::viewerSpawned)" );
   ACE_DEBUG(( LM_DEBUG, "vid: %d\n", vid ));
   Protocol::PMessage* msg;
   if( proto.getMsgType( VSPWD, msg ) == Protocol::P_OK )
   {
      msg->set( &vid );
      proto.sendMsg( VSPWD );
   }
   return 0;
}

int TangoLinkHandler::
viewerClosed( const Header::VID& vid )
{
   ACE_TRACE( "(TangoLinkHandler::viewerClosed)" );
   ACE_DEBUG(( LM_DEBUG, "vid: %d\n", vid ));
   Protocol::PMessage* msg;
   if( proto.getMsgType( VKILD, msg ) == Protocol::P_OK )
   {
      msg->set( &vid );
      proto.sendMsg( VKILD );
   }
   return 0;
}
