/*
 * File:       TangoLinkHandler.h
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    Mon Apr 21 1997
 * Revision:   $Revision: 1.5 $
 * Updated:    $Date: 1998/04/25 06:15:00 $
 * Copyright 1997 NPAC, Syracuse University
 *
 */

#ifndef _TANGOLINKHANDLER_H
#define _TANGOLINKHANDLER_H

#include <ace/Event_Handler.h>
#include <ace/SOCK_Stream.h>

#include "ControlProto.h"
#include "ControlMessage.h"
#include "Event.h"

#include <MsgHeader.h>
#include <FixedVector.h>
#include <ProtocolInterface.h>

class ControlCenter;

class TangoLinkHandler : public ACE_Event_Handler
{
   public:
      typedef ACE_SOCK_Stream IPC_SAP;
      typedef MsgHeader<IPC_SAP> Header;
      typedef ControlMessage<Header> Message;
      typedef ProtocolInterface<Message, FixedVector<Message*> > Protocol;

      TangoLinkHandler( ControlCenter& ctrl, int handle );
      int handle_input( ACE_HANDLE );
      int handle_close( ACE_HANDLE, ACE_Reactor_Mask );
      ACE_HANDLE get_handle( void ) const;
      void set_handle( ACE_HANDLE );
      void ping( void );
      void sendSynchro( void );
      void sendSelArea( void );
      int send_msg( Header& header, ACE_Message_Block& mb );
      int viewerSpawned( const Header::VID& );
      int viewerClosed( const Header::VID& );
      
   private:
      static Protocol::PSTATUS viewerMsgHandler( Protocol::SAP& sap,
						 Header&,
						 ControlCenter* );
      
      Protocol proto;
      ControlCenter& control;
};

inline ACE_HANDLE TangoLinkHandler::
get_handle( void ) const
{
   ACE_TRACE( "(TangoLinkHandler::get_handle)" );
   return proto.getSAP().get_handle();
}

inline void TangoLinkHandler::
set_handle( ACE_HANDLE fd )
{
   ACE_TRACE( "(TangoLinkHandler::set_handle)" );
   proto.getSAP().set_handle( fd );
}

inline int TangoLinkHandler::
handle_close( ACE_HANDLE, ACE_Reactor_Mask )
{
   ACE_TRACE( "(TangoLinkHandler::handle_close)" );
   return close( proto.getSAP().get_handle() );
}

inline void TangoLinkHandler::
sendSynchro( void )
{
   ACE_TRACE( "(TangoLinkHandler::sendSynchro)" );
   proto.sendMsg( CSYNCI );
   proto.sendMsg( AUPDT );
}

inline void TangoLinkHandler::
sendSelArea( void )
{
   ACE_TRACE( "(TangoLinkHandler::sendSynchro)" );
   proto.sendMsg( AUPDT );
}

#endif
