/*
 * File:       ViewerLinkHandler.C
 * Author:     Pawel Roman (proman@npac.syr.edu)
 * Created:    Sun Apr 27 1997
 * Revision:   $Revision: 1.3 $
 * Updated:    $Date: 1997/08/10 22:49:34 $
 * Copyright 1997 NPAC, Syracuse University
 *
 */

#include "ViewerLinkHandler.h"
#include "ControlCenter.h"

ViewerLinkHandler::
ViewerLinkHandler( ControlCenter& ctrl, ACE_HANDLE fd )
   : control( ctrl ), header( 0 )
{
   ACE_TRACE( "(ViewerLinkHandler::ViewerLinkHandler)" );
   sap.set_handle( fd );
}

int ViewerLinkHandler::
handle_input( ACE_HANDLE )
{
   ACE_TRACE( "(ViewerLinkHandler::handle_input)" );
   if( header.read( sap ) == MsgHeader<IPC_SAP>::LINK_OK )
   {
      MsgHeader<IPC_SAP>::LengthType msg_len = header.length();
      ACE_Message_Block mb( msg_len );
      if( sap.recv_n( mb.rd_ptr(), msg_len ) == msg_len )
      {
	 ACE_DEBUG(( LM_DEBUG, "received %d bytes\n", mb.size() ));
	 control.sendToTango( header, mb, this );
      }
      else
	 ACE_ERROR_RETURN(( LM_ERROR,
			    "[!](ViewerLinkHandler::handle_input) recv_n error\n" ),
			  -1);
   }
   else
      control.viewerClosed( this );
   
   return 0;
}

int ViewerLinkHandler::
send_msg( MsgHeader<IPC_SAP>& header, ACE_Message_Block& mb )
{
   ACE_TRACE( "(ViewerLinkHandler::send_msg)" );
   if( header.write( sap ) == MsgHeader<IPC_SAP>::LINK_OK )
   {
      if( sap.send_n( mb.rd_ptr(), header.length() ) != mb.size() )
	 ACE_ERROR_RETURN(( LM_ERROR,
			    "[!](ViewerLinkHandler::send_msg) send_n error\n" ), -1 );
   }
   else
      ACE_ERROR_RETURN(( LM_ERROR,
			 "[!](ViewerLinkHandler::send_msg) header.write error\n" ), -1 );
   
   return 0;
}
