/*
 * File         : msgAUPDT.h
 * Author       : Pawel Roman
 * Created      : Fri Aug  8 21:24:40 1997
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1997/08/10 22:49:12 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGAUPDT_H
#define __MSGAUPDT_H

#include "ControlProto.h"
#include "ControlMessage.h"

template<class HEADER>
class msgAUPDT : public ControlMessage<HEADER>
{
   public:
      msgAUPDT( ControlCenter* cc );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );

   private:
      ControlCenter::AreaParams areaPrms;
};

template<class HEADER>
inline msgAUPDT<HEADER>::
msgAUPDT( ControlCenter* cc )
   : ControlMessage<HEADER>( Header( AUPDT, 0, sizeof areaPrms ), cc )
{ 
   ACE_TRACE( "(msgAUPDT::msgAUPDT)" );
}

template<class HEADER>
inline msgAUPDT<HEADER>::MRESULT msgAUPDT<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgAUPDT::read)" );
   if( sap.recv_n( &areaPrms, sizeof areaPrms ) == header.length() )
      return OK_RES;
   else
      return END_RES;
}

template<class HEADER>
inline msgAUPDT<HEADER>::MRESULT msgAUPDT<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgAUPDT::write)" );
   control->getSelArea( areaPrms );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &areaPrms, sizeof areaPrms ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgAUPDT<HEADER>::MRESULT msgAUPDT<HEADER>::
set( const void* ap ) 
{ 
   ACE_TRACE( "(msgAUPDT::set)" );
   memcpy( areaPrms, ap, sizeof areaPrms );
   return OK_RES;
}

template<class HEADER>
inline void msgAUPDT<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgAUPDT::execute)" );
   control->setSelArea( areaPrms );
}

#endif
