/*
 * File         : msgCSYNCI.h
 * Author       : Pawel Roman
 * Created      : Tue Aug  5 19:47:14 1997
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/04/25 06:15:01 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGCSYNCI_H
#define __MSGCSYNCI_H

#include "ControlProto.h"
#include "ControlMessage.h"

template<class HEADER>
class msgCSYNCI : public ControlMessage<HEADER>
{
   public:
      msgCSYNCI( ControlCenter* cc );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      bool initialState;
      HEADER::VID vidRep;
};

template<class HEADER>
inline msgCSYNCI<HEADER>::
msgCSYNCI( ControlCenter* cc )
   : ControlMessage<HEADER>( Header( CSYNCI, 0, sizeof vidRep ), cc ),
     initialState( true )
{ 
   ACE_TRACE( "(msgCSYNCI::msgCSYNCI)" );
}

template<class HEADER>
inline msgCSYNCI<HEADER>::MRESULT msgCSYNCI<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgCSYNCI::read)" );
   if( sap.recv_n( &vidRep, sizeof vidRep ) == header.length() )
      return OK_RES;
   else
     {    
       ACE_DEBUG(( LM_DEBUG, "(msgCSYNCI::read) size: %d, len: %d\n",
		   sizeof vidRep, header.length() ));
       return END_RES;
     }
}

template<class HEADER>
inline msgCSYNCI<HEADER>::MRESULT msgCSYNCI<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgCSYNCI::write)" );
   control->setVIDRep( vidRep );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &vidRep, sizeof vidRep ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgCSYNCI<HEADER>::MRESULT msgCSYNCI<HEADER>::
set( const void* vr ) 
{ 
   ACE_TRACE( "(msgCSYNCI::set)" );
   vidRep = *( HEADER::VID* )vr;
   return OK_RES;
}

template<class HEADER>
inline void msgCSYNCI<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgCSYNCI::execute)" );
   if( initialState )
   {
      control->synchronize( vidRep );
      initialState = false;
   }
}

#endif

