/*
 * File         : msgVKILD.h
 * Author       : Pawel Roman
 * Created      : Tue Aug  5 19:47:14 1997
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1997/08/10 22:49:13 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGVKILD_H
#define __MSGVKILD_H

#include "ControlProto.h"
#include "ControlMessage.h"

template<class HEADER>
class msgVKILD : public ControlMessage<HEADER>
{
   public:
      msgVKILD( ControlCenter* cc );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      HEADER::VID vid;
};

template<class HEADER>
inline msgVKILD<HEADER>::
msgVKILD( ControlCenter* cc )
   : ControlMessage<HEADER>( Header( VKILD, 0, sizeof vid ), cc )
{ 
   ACE_TRACE( "(msgVKILD::msgVKILD)" );
}

template<class HEADER>
inline msgVKILD<HEADER>::MRESULT msgVKILD<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgVKILD::read)" );
   if( sap.recv_n( &vid, sizeof vid ) == header.length() )
      return OK_RES;
   else
      return END_RES;
}

template<class HEADER>
inline msgVKILD<HEADER>::MRESULT msgVKILD<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgVKILD::write)" );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &vid, sizeof vid ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgVKILD<HEADER>::MRESULT msgVKILD<HEADER>::
set( const void* viewerID ) 
{ 
   ACE_TRACE( "(msgVKILD::set)" );
   vid = *( HEADER::VID* )viewerID;
   return OK_RES;
}

template<class HEADER>
inline void msgVKILD<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgVKILD::execute)" );
   control->viewerClosed( vid );
}

#endif

