/*
 * File         : msgVSPWD.h
 * Author       : Pawel Roman
 * Created      : Tue Aug  5 19:47:14 1997
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1997/08/10 22:49:14 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#ifndef __MSGVSPWD_H
#define __MSGVSPWD_H

#include "ControlProto.h"
#include "ControlMessage.h"

template<class HEADER>
class msgVSPWD : public ControlMessage<HEADER>
{
   public:
      msgVSPWD( ControlCenter* cc );
      MRESULT read( const SAP& sap, const Header& header );
      MRESULT write( const SAP& sap );
      MRESULT set( const void* params = NULL );
      void execute( void );
  
   private:
      HEADER::VID vid;
};

template<class HEADER>
inline msgVSPWD<HEADER>::
msgVSPWD( ControlCenter* cc )
   : ControlMessage<HEADER>( Header( VSPWD, 0, sizeof vid ), cc )
{ 
   ACE_TRACE( "(msgVSPWD::msgVSPWD)" );
}

template<class HEADER>
inline msgVSPWD<HEADER>::MRESULT msgVSPWD<HEADER>::
read( const SAP& sap, const Header& header )
{
   ACE_TRACE( "(msgVSPWD::read)" );
   if( sap.recv_n( &vid, sizeof vid ) == header.length() )
      return OK_RES;
   else
      return END_RES;
}

template<class HEADER>
inline msgVSPWD<HEADER>::MRESULT msgVSPWD<HEADER>::
write( const SAP& sap ) 
{ 
   ACE_TRACE( "(msgVSPWD::write)" );
   if( header.write( sap ) == Header::LINK_OK )
      if( sap.send_n( &vid, sizeof vid ) == header.length() )
	 return OK_RES;
   
   return END_RES;
}

template<class HEADER>
inline msgVSPWD<HEADER>::MRESULT msgVSPWD<HEADER>::
set( const void* viewerID ) 
{ 
   ACE_TRACE( "(msgVSPWD::set)" );
   vid = *( HEADER::VID* )viewerID;
   return OK_RES;
}

template<class HEADER>
inline void msgVSPWD<HEADER>::
execute( void )
{
   ACE_TRACE( "(msgVSPWD::execute)" );
   control->spawnNewViewer( vid );
}

#endif

